package su.a71.new_soviet.items;

import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.DoorBlock;
import net.minecraft.block.HorizontalFacingBlock;
import net.minecraft.block.enums.DoorHinge;
import net.minecraft.block.enums.DoubleBlockHalf;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.*;
import net.minecraft.sound.SoundCategory;
import net.minecraft.stat.Stats;
import net.minecraft.util.ActionResult;
import net.minecraft.util.UseAction;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.world.World;
import su.a71.new_soviet.NewSoviet;
import su.a71.new_soviet.blocks.WindowBlock;
import su.a71.new_soviet.registration.NSE_Blocks;
import su.a71.new_soviet.registration.NSE_Sounds;

public class WhitewashedBrushItem extends Item {

    public WhitewashedBrushItem(Settings settings) {
        super(settings);
    }

    public UseAction getUseAction(ItemStack stack) {
        return UseAction.BRUSH;
    }

    public void remove_whitewash(PlayerEntity player) {
        player.getItemCooldownManager().set(this, 5);
        player.incrementStat(Stats.USED.getOrCreateStat(this));
        if (!player.isCreative()) {
            player.getInventory().setStack(player.getInventory().selectedSlot, Items.BRUSH.getDefaultStack());
        }
    }

    public void particles(World world, BlockPos pos) {
        world.addBlockBreakParticles(pos, NSE_Blocks.WHITEWASH.getDefaultState());
    }

    public void paint_sound(World world, BlockPos pos, Entity entity) {
        world.playSound(entity, pos, NSE_Sounds.BRUSH_PAINTING_SOUND, SoundCategory.PLAYERS, 1, (float) NewSoviet.RANDOM.nextBetween(8, 12) / 10f);
    }

    @Override
    public ActionResult useOnBlock(ItemUsageContext context) {
        World world = context.getWorld();
        BlockPos pos = context.getBlockPos();
        BlockState state = world.getBlockState(pos);
        Block block = state.getBlock();
        PlayerEntity player = context.getPlayer();

        if (block == NSE_Blocks.CONCRETE) {
            if (NewSoviet.RANDOM.nextBetween(1, 4) != 4) {
                world.setBlockState(pos, NSE_Blocks.WHITEWASH.getDefaultState());
            } else {
                world.setBlockState(pos, NSE_Blocks.CRACKED_WHITEWASH.getDefaultState());
            }
            particles(world, pos);
            paint_sound(world, pos, player);
            remove_whitewash(player);
            return ActionResult.SUCCESS;
        }
        if (block == NSE_Blocks.CRACKED_WHITEWASH) {
            world.setBlockState(pos, NSE_Blocks.WHITEWASH.getDefaultState());
            particles(world, pos);
            paint_sound(world, pos, player);
            remove_whitewash(player);
            return ActionResult.SUCCESS;
        }
        if (block == NSE_Blocks.VERY_CRACKED_WHITEWASH) {
            if (NewSoviet.RANDOM.nextBetween(1, 4) != 4) {
                world.setBlockState(pos, NSE_Blocks.CRACKED_WHITEWASH.getDefaultState());
            } else {
                world.setBlockState(pos, NSE_Blocks.WHITEWASH.getDefaultState());
            }
            particles(world, pos);
            paint_sound(world, pos, player);
            remove_whitewash(player);
            return ActionResult.SUCCESS;
        }

        if (block instanceof WindowBlock) {
            int window_type = world.getBlockState(pos).get(WindowBlock.WINDOW_TYPE);
            boolean broken = world.getBlockState(pos).get(WindowBlock.BROKEN);
            Direction facing = world.getBlockState(pos).get(HorizontalFacingBlock.FACING);
            boolean closed = world.getBlockState(pos).get(WindowBlock.CLOSED);
            int broken_glass_type = world.getBlockState(pos).get(WindowBlock.BROKEN_GLASS_TYPE);
            boolean waterlogged = world.getBlockState(pos).get(WindowBlock.WATERLOGGED);
            DoorHinge hinge = world.getBlockState(pos).get(WindowBlock.HINGE);
            WindowBlock windowBlock = (WindowBlock) block;
            boolean pane = windowBlock.getPane();
            BlockState newWindowState = NSE_Blocks.WHITEWASHED_WINDOW.getDefaultState();
            if (pane) {
                newWindowState = NSE_Blocks.WHITEWASHED_PANE_WINDOW.getDefaultState();
            }
            world.setBlockState(pos, newWindowState.with(WindowBlock.WINDOW_TYPE, window_type)
                    .with(WindowBlock.BROKEN, broken)
                    .with(HorizontalFacingBlock.FACING, facing)
                    .with(WindowBlock.CLOSED, closed)
                    .with(WindowBlock.BROKEN_GLASS_TYPE, broken_glass_type)
                    .with(WindowBlock.WATERLOGGED, waterlogged)
                    .with(WindowBlock.HINGE, hinge));
            particles(world, pos);
            paint_sound(world, pos, player);
            remove_whitewash(player);
            return ActionResult.SUCCESS;
        }

        if (block == NSE_Blocks.CHISELED_BIRCH_DOOR) {
            DoubleBlockHalf half = world.getBlockState(pos).get(DoorBlock.HALF);
            BlockPos pos1 = pos;
            BlockPos pos2 = pos.up();
            if (half == DoubleBlockHalf.UPPER) {
                pos1 = pos.down();
                pos2 = pos;
            }
            boolean open = world.getBlockState(pos).get(DoorBlock.OPEN);
            Direction facing = world.getBlockState(pos).get(DoorBlock.FACING);
            boolean powered = world.getBlockState(pos).get(DoorBlock.POWERED);
            DoorHinge hinge = world.getBlockState(pos).get(DoorBlock.HINGE);
            BlockState door_state = NSE_Blocks.WHITE_CHISELED_BIRCH_DOOR.getDefaultState()
                    .with(DoorBlock.POWERED, powered)
                    .with(DoorBlock.FACING, facing)
                    .with(DoorBlock.OPEN, open)
                    .with(DoorBlock.HINGE, hinge);
            world.removeBlock(pos1, false);
            world.removeBlock(pos2, true);

            world.setBlockState(pos1, door_state.with(DoorBlock.HALF, DoubleBlockHalf.LOWER));
            world.setBlockState(pos2, door_state.with(DoorBlock.HALF, DoubleBlockHalf.UPPER));
            world.setBlockState(pos1, door_state.with(DoorBlock.HALF, DoubleBlockHalf.LOWER));
            paint_sound(world, pos, player);
            remove_whitewash(player);
            return ActionResult.SUCCESS;
        }
        return super.useOnBlock(context);

    }
}

